@@ -6,7 +6,7 @@
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
  *
- *      http://www.apache.org/licenses/LICENSE-2.0
+ *      https://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -20,127 +20,53 @@ package org.apache.commons.dbcp2;
 import java.sql.Connection;
 import java.sql.ResultSet;
 import java.sql.SQLException;
+import java.sql.SQLTimeoutException;
 import java.sql.SQLWarning;
 import java.sql.Statement;
 
 /**
  * A dummy {@link Statement}, for testing purposes.
- *
- * @author Rodney Waldhoff
- * @author Dirk Verbeeck
- * @version $Id: TesterStatement.java 1649424 2015-01-04 21:15:33Z tn $
  */
-public class TesterStatement implements Statement {
-    public TesterStatement(Connection conn) {
-        _connection = conn;
+public class TesterStatement extends AbandonedTrace implements Statement {
+
+    protected final Connection connection;
+    protected boolean open = true;
+    protected final long rowsUpdated = 1;
+    protected final boolean executeResponse = true;
+    protected int maxFieldSize = 1024;
+    protected long maxRows = 1024;
+    protected boolean escapeProcessing;
+    protected int queryTimeout = 1000;
+    protected String cursorName;
+    protected int fetchDirection = 1;
+    protected int fetchSize = 1;
+    protected int resultSetConcurrency = 1;
+    protected int resultSetType = 1;
+    private int resultSetHoldability = 1;
+    protected ResultSet resultSet;
+    protected boolean sqlExceptionOnClose;
+
+    public TesterStatement(final Connection connection) {
+        this.connection = connection;
     }
 
-    public TesterStatement(Connection conn, int resultSetType, int resultSetConcurrency) {
-        _connection = conn;
-        _resultSetType = resultSetType;
-        _resultSetConcurrency = resultSetConcurrency;
+    public TesterStatement(final Connection connection, final int resultSetType, final int resultSetConcurrency) {
+        this.connection = connection;
+        this.resultSetType = resultSetType;
+        this.resultSetConcurrency = resultSetConcurrency;
     }
 
-    protected Connection _connection = null;
-    protected boolean _open = true;
-    protected int _rowsUpdated = 1;
-    protected boolean _executeResponse = true;
-    protected int _maxFieldSize = 1024;
-    protected int _maxRows = 1024;
-    protected boolean _escapeProcessing = false;
-    protected int _queryTimeout = 1000;
-    protected String _cursorName = null;
-    protected int _fetchDirection = 1;
-    protected int _fetchSize = 1;
-    protected int _resultSetConcurrency = 1;
-    protected int _resultSetType = 1;
-    protected ResultSet _resultSet = null;
+    public TesterStatement(final Connection connection, final int resultSetType, final int resultSetConcurrency,
+            final int resultSetHoldability) {
+        this.connection = connection;
+        this.resultSetType = resultSetType;
+        this.resultSetConcurrency = resultSetConcurrency;
+        this.resultSetHoldability = resultSetHoldability;
+    }
 
     @Override
-    public ResultSet executeQuery(String sql) throws SQLException {
+    public void addBatch(final String sql) throws SQLException {
         checkOpen();
-        if("null".equals(sql)) {
-            return null;
-        }
-        if("invalid".equals(sql)) {
-            throw new SQLException("invalid query");
-        }
-        if ("broken".equals(sql)) {
-            throw new SQLException("broken connection");
-        }
-        if("select username".equals(sql)) {
-            String username = ((TesterConnection) _connection).getUsername();
-            Object[][] data = {{username}};
-            return new TesterResultSet(this, data);
-        } else {
-            // Simulate timeout if queryTimout is set to less than 5 seconds
-            if (_queryTimeout > 0 && _queryTimeout < 5) {
-                throw new SQLException("query timeout");
-            }
-            return new TesterResultSet(this);
-        }
-    }
-
-    @Override
-    public int executeUpdate(String sql) throws SQLException {
-        checkOpen();
-        return _rowsUpdated;
-    }
-
-    @Override
-    public void close() throws SQLException {
-        // calling close twice has no effect
-        if (!_open) {
-            return;
-        }
-
-        _open = false;
-        if (_resultSet != null) {
-            _resultSet.close();
-            _resultSet = null;
-        }
-    }
-
-    @Override
-    public int getMaxFieldSize() throws SQLException {
-        checkOpen();
-        return _maxFieldSize;
-    }
-
-    @Override
-    public void setMaxFieldSize(int max) throws SQLException {
-        checkOpen();
-        _maxFieldSize = max;
-    }
-
-    @Override
-    public int getMaxRows() throws SQLException {
-        checkOpen();
-        return _maxRows;
-    }
-
-    @Override
-    public void setMaxRows(int max) throws SQLException {
-        checkOpen();
-        _maxRows = max;
-    }
-
-    @Override
-    public void setEscapeProcessing(boolean enable) throws SQLException {
-        checkOpen();
-        _escapeProcessing = enable;
-    }
-
-    @Override
-    public int getQueryTimeout() throws SQLException {
-        checkOpen();
-        return _queryTimeout;
-    }
-
-    @Override
-    public void setQueryTimeout(int seconds) throws SQLException {
-        checkOpen();
-        _queryTimeout = seconds;
     }
 
     @Override
@@ -148,10 +74,22 @@ public class TesterStatement implements Statement {
         checkOpen();
     }
 
+    protected void checkOpen() throws SQLException {
+        if (!open) {
+            throw new SQLException("Connection is closed.");
+        }
+    }
+
+    protected void checkQueryTimeout() throws SQLTimeoutException {
+        if (queryTimeout > 0 && queryTimeout < 5) {
+            // Simulate timeout if queryTimout is set to less than 5 seconds
+            throw new SQLTimeoutException("query timeout " + queryTimeout);
+        }
+    }
+
     @Override
-    public SQLWarning getWarnings() throws SQLException {
+    public void clearBatch() throws SQLException {
         checkOpen();
-        return null;
     }
 
     @Override
@@ -160,85 +98,53 @@ public class TesterStatement implements Statement {
     }
 
     @Override
-    public void setCursorName(String name) throws SQLException {
-        checkOpen();
-        _cursorName = name;
+    public void close() throws SQLException {
+        if (sqlExceptionOnClose) {
+            throw new SQLException("TestSQLExceptionOnClose");
+        }
+
+        // calling close twice has no effect
+        if (!open) {
+            return;
+        }
+
+        open = false;
+        if (resultSet != null) {
+            resultSet.close();
+            resultSet = null;
+        }
     }
 
     @Override
-    public boolean execute(String sql) throws SQLException {
+    public void closeOnCompletion() throws SQLException {
+        throw new SQLException("Not implemented.");
+    }
+
+    @Override
+    public boolean execute(final String sql) throws SQLException {
         checkOpen();
-        if("invalid".equals(sql)) {
+        if ("invalid".equals(sql)) {
             throw new SQLException("invalid query");
         }
-        return _executeResponse;
+        return executeResponse;
     }
 
     @Override
-    public ResultSet getResultSet() throws SQLException {
-        checkOpen();
-        if (_resultSet == null) {
-            _resultSet = new TesterResultSet(this);
-        }
-        return _resultSet;
+    public boolean execute(final String sql, final int autoGeneratedKeys)
+        throws SQLException {
+        throw new SQLException("Not implemented.");
     }
 
     @Override
-    public int getUpdateCount() throws SQLException {
-        checkOpen();
-        return _rowsUpdated;
+    public boolean execute(final String sql, final int[] columnIndexes)
+        throws SQLException {
+        throw new SQLException("Not implemented.");
     }
 
     @Override
-    public boolean getMoreResults() throws SQLException {
-        checkOpen();
-        return false;
-    }
-
-    @Override
-    public void setFetchDirection(int direction) throws SQLException {
-        checkOpen();
-        _fetchDirection = direction;
-    }
-
-    @Override
-    public int getFetchDirection() throws SQLException {
-        checkOpen();
-        return _fetchDirection;
-    }
-
-    @Override
-    public void setFetchSize(int rows) throws SQLException {
-        checkOpen();
-        _fetchSize = rows;
-    }
-
-    @Override
-    public int getFetchSize() throws SQLException {
-        checkOpen();
-        return _fetchSize;
-    }
-
-    @Override
-    public int getResultSetConcurrency() throws SQLException {
-        checkOpen();
-        return _resultSetConcurrency;
-    }
-
-    @Override
-    public int getResultSetType() throws SQLException {
-        checkOpen();
-        return _resultSetType;
-    }
-
-    @Override
-    public void addBatch(String sql) throws SQLException {
-        checkOpen();
-    }
-
-    @Override
-    public void clearBatch() throws SQLException {
-        checkOpen();
+    public boolean execute(final String sql, final String[] columnNames)
+        throws SQLException {
+        throw new SQLException("Not implemented.");
     }
 
     @Override
@@ -248,87 +154,185 @@ public class TesterStatement implements Statement {
     }
 
     @Override
-    public Connection getConnection() throws SQLException {
+    public long[] executeLargeBatch() throws SQLException {
         checkOpen();
-        return _connection;
-    }
-
-    protected void checkOpen() throws SQLException {
-        if(!_open) {
-            throw new SQLException("Connection is closed.");
-        }
+        return new long[0];
     }
 
     @Override
-    public boolean getMoreResults(int current) throws SQLException {
+    public long executeLargeUpdate(final String sql) throws SQLException {
+        checkOpen();
+        return rowsUpdated;
+    }
+
+    @Override
+    public long executeLargeUpdate(final String sql, final int autoGeneratedKeys) throws SQLException {
         throw new SQLException("Not implemented.");
     }
 
+    @Override
+    public long executeLargeUpdate(final String sql, final int[] columnIndexes) throws SQLException {
+        throw new SQLException("Not implemented.");
+    }
+
+    @Override
+    public long executeLargeUpdate(final String sql, final String[] columnNames) throws SQLException {
+        throw new SQLException("Not implemented.");
+    }
+
+    @Override
+    public ResultSet executeQuery(final String sql) throws SQLException {
+        checkOpen();
+        switch (sql) {
+        case "null":
+            return null;
+        case "invalid":
+            throw new SQLException("invalid query");
+        case "broken":
+            throw new SQLException("broken connection");
+        case "select username": {
+            final String userName = ((TesterConnection) connection).getUserName();
+            final Object[][] data = { { userName } };
+            return new TesterResultSet(this, data);
+        }
+        default:
+            break;
+        }
+        checkQueryTimeout();
+        return new TesterResultSet(this);
+    }
+
+    @Override
+    public int executeUpdate(final String sql) throws SQLException {
+        checkOpen();
+        return (int) rowsUpdated;
+    }
+
+    @Override
+    public int executeUpdate(final String sql, final int autoGeneratedKeys) throws SQLException {
+        throw new SQLException("Not implemented.");
+    }
+
+    @Override
+    public int executeUpdate(final String sql, final int[] columnIndexes) throws SQLException {
+        throw new SQLException("Not implemented.");
+    }
+
+    @Override
+    public int executeUpdate(final String sql, final String[] columnNames) throws SQLException {
+        throw new SQLException("Not implemented.");
+    }
+
+    @Override
+    public Connection getConnection() throws SQLException {
+        checkOpen();
+        return connection;
+    }
+
+    @Override
+    public int getFetchDirection() throws SQLException {
+        checkOpen();
+        return fetchDirection;
+    }
+
+    @Override
+    public int getFetchSize() throws SQLException {
+        checkOpen();
+        return fetchSize;
+    }
+
     @Override
     public ResultSet getGeneratedKeys() throws SQLException {
         return new TesterResultSet(this);
     }
 
     @Override
-    public int executeUpdate(String sql, int autoGeneratedKeys)
-        throws SQLException {
+    public long getLargeMaxRows() throws SQLException {
+        checkOpen();
+        return maxRows;
+    }
+
+    @Override
+    public long getLargeUpdateCount() throws SQLException {
+        checkOpen();
+        return rowsUpdated;
+    }
+
+    @Override
+    public int getMaxFieldSize() throws SQLException {
+        checkOpen();
+        return maxFieldSize;
+    }
+
+    @Override
+    public int getMaxRows() throws SQLException {
+        checkOpen();
+        return (int) maxRows;
+    }
+
+    @Override
+    public boolean getMoreResults() throws SQLException {
+        checkOpen();
+        return false;
+    }
+
+    @Override
+    public boolean getMoreResults(final int current) throws SQLException {
         throw new SQLException("Not implemented.");
     }
 
     @Override
-    public int executeUpdate(String sql, int columnIndexes[])
-        throws SQLException {
-        throw new SQLException("Not implemented.");
+    public int getQueryTimeout() throws SQLException {
+        checkOpen();
+        return queryTimeout;
     }
 
     @Override
-    public int executeUpdate(String sql, String columnNames[])
-        throws SQLException {
-        throw new SQLException("Not implemented.");
+    public ResultSet getResultSet() throws SQLException {
+        checkOpen();
+        if (resultSet == null) {
+            resultSet = new TesterResultSet(this);
+        }
+        return resultSet;
     }
 
     @Override
-    public boolean execute(String sql, int autoGeneratedKeys)
-        throws SQLException {
-        throw new SQLException("Not implemented.");
-    }
-
-    @Override
-    public boolean execute(String sql, int columnIndexes[])
-        throws SQLException {
-        throw new SQLException("Not implemented.");
-    }
-
-    @Override
-    public boolean execute(String sql, String columnNames[])
-        throws SQLException {
-        throw new SQLException("Not implemented.");
+    public int getResultSetConcurrency() throws SQLException {
+        checkOpen();
+        return resultSetConcurrency;
     }
 
     @Override
     public int getResultSetHoldability() throws SQLException {
         checkOpen();
-        throw new SQLException("Not implemented.");
-    }
-
-
-    @Override
-    public boolean isWrapperFor(Class<?> iface) throws SQLException {
-        throw new SQLException("Not implemented.");
+        return resultSetHoldability;
     }
 
     @Override
-    public <T> T unwrap(Class<T> iface) throws SQLException {
-        throw new SQLException("Not implemented.");
+    public int getResultSetType() throws SQLException {
+        checkOpen();
+        return resultSetType;
+    }
+
+    @Override
+    public int getUpdateCount() throws SQLException {
+        checkOpen();
+        return (int) rowsUpdated;
+    }
+
+    @Override
+    public SQLWarning getWarnings() throws SQLException {
+        checkOpen();
+        return null;
     }
 
     @Override
     public boolean isClosed() throws SQLException {
-        return !_open;
+        return !open;
     }
 
     @Override
-    public void setPoolable(boolean poolable) throws SQLException {
+    public boolean isCloseOnCompletion() throws SQLException {
         throw new SQLException("Not implemented.");
     }
 
@@ -337,13 +341,74 @@ public class TesterStatement implements Statement {
         throw new SQLException("Not implemented.");
     }
 
+    public boolean isSqlExceptionOnClose() {
+        return sqlExceptionOnClose;
+    }
+
     @Override
-    public void closeOnCompletion() throws SQLException {
+    public boolean isWrapperFor(final Class<?> iface) throws SQLException {
         throw new SQLException("Not implemented.");
     }
 
     @Override
-    public boolean isCloseOnCompletion() throws SQLException {
+    public void setCursorName(final String name) throws SQLException {
+        checkOpen();
+        this.cursorName = name;
+    }
+
+    @Override
+    public void setEscapeProcessing(final boolean enable) throws SQLException {
+        checkOpen();
+        this.escapeProcessing = enable;
+    }
+
+    @Override
+    public void setFetchDirection(final int direction) throws SQLException {
+        checkOpen();
+        this.fetchDirection = direction;
+    }
+
+    @Override
+    public void setFetchSize(final int rows) throws SQLException {
+        checkOpen();
+        this.fetchSize = rows;
+    }
+
+    @Override
+    public void setLargeMaxRows(final long max) throws SQLException {
+        checkOpen();
+        this.maxRows = max;
+    }
+
+    @Override
+    public void setMaxFieldSize(final int max) throws SQLException {
+        checkOpen();
+        this.maxFieldSize = max;
+    }
+
+    @Override
+    public void setMaxRows(final int max) throws SQLException {
+        checkOpen();
+        this.maxRows = max;
+    }
+
+    @Override
+    public void setPoolable(final boolean poolable) throws SQLException {
+        throw new SQLException("Not implemented.");
+    }
+
+    @Override
+    public void setQueryTimeout(final int seconds) throws SQLException {
+        checkOpen();
+        this.queryTimeout = seconds;
+    }
+
+    public void setSqlExceptionOnClose(final boolean sqlExceptionOnClose) {
+        this.sqlExceptionOnClose = sqlExceptionOnClose;
+    }
+
+    @Override
+    public <T> T unwrap(final Class<T> iface) throws SQLException {
         throw new SQLException("Not implemented.");
     }
 }
